@@ -14,6 +14,8 @@ use debcargo::{
 
 #[cfg(feature = "update-dependencies")]
 use debcargo::update_dependencies::update_dependencies;
+#[cfg(feature = "update-dependencies")]
+use Opt::UpdateDependencies;
 use Opt::{BuildOrder, DebDependencies, DebSrcName, Extract, Package, Update};
 
 #[test]
@@ -75,7 +77,7 @@ fn real_main() -> Result<()> {
             Ok(())
         }
         #[cfg(feature = "update-dependencies")]
-        UpdateDependencies { args } => update_dependencies(args),
+        UpdateDependencies { args } => update_dependencies(&args),
     }
 }
 
