@@ -1,9 +1,12 @@
+use serde::de::IgnoredAny;
 use serde::Deserialize;
 use toml;
 
-use crate::errors::*;
+use crate::errors::Result;
 
+use std::borrow::Cow;
 use std::collections::HashMap;
+use std::ffi::OsStr;
 use std::fs::File;
 use std::io::Read;
 use std::path::{Path, PathBuf};
@@ -11,6 +14,8 @@ use std::path::{Path, PathBuf};
 pub const RUST_MAINT: &str =
     "Debian Rust Maintainers <pkg-rust-maintainers@alioth-lists.debian.net>";
 
+pub const DEFAULT_REPACK_SUFFIX: &str = "dfsg";
+
 #[derive(Deserialize, Debug, Clone)]
 #[serde(default)]
 pub struct Config {
@@ -20,6 +25,7 @@ pub struct Config {
     pub overlay: Option<PathBuf>,
     pub excludes: Option<Vec<String>>,
     pub whitelist: Option<Vec<String>>,
+    pub repack_suffix: Option<String>,
     pub allow_prerelease_deps: bool,
     pub crate_src_path: Option<PathBuf>,
     pub summary: Option<String>,
@@ -33,7 +39,7 @@ pub struct Config {
     pub packages: HashMap<String, PackageOverride>,
 
     #[serde(flatten)]
-    pub unknown_fields: HashMap<String, toml::Value>,
+    pub unknown_fields: HashMap<String, IgnoredAny>,
 }
 
 #[derive(Deserialize, Debug, Clone, Default)]
@@ -50,10 +56,11 @@ pub struct SourceOverride {
     skip_nocheck: Option<bool>,
 
     #[serde(flatten)]
-    pub unknown_fields: HashMap<String, toml::Value>,
+    pub unknown_fields: HashMap<String, IgnoredAny>,
 }
 
 impl SourceOverride {
+    #[expect(clippy::too_many_arguments)]
     pub fn new(
         section: Option<String>,
         policy: Option<String>,
@@ -101,7 +108,7 @@ pub struct PackageOverride {
     test_depends: Option<Vec<String>>,
 
     #[serde(flatten)]
-    pub unknown_fields: HashMap<String, toml::Value>,
+    pub unknown_fields: HashMap<String, IgnoredAny>,
 }
 
 impl Default for Config {
@@ -112,6 +119,7 @@ impl Default for Config {
             semver_suffix: false,
             overlay: None,
             excludes: None,
+            repack_suffix: None,
             whitelist: None,
             allow_prerelease_deps: false,
             crate_src_path: None,
@@ -144,13 +152,19 @@ impl Config {
 
         if let Some(ref source) = config.source {
             for field in source.unknown_fields.keys() {
-                unknown_fields.push(format!("source.{}", field));
+                unknown_fields.push(format!("source.{field}"));
+            }
+        }
+
+        for field in config.packages.keys() {
+            if PackageKey::from_key(field).is_none() {
+                unknown_fields.push(format!("packages.{field}"));
             }
         }
 
         for (name, package) in &config.packages {
             for field in package.unknown_fields.keys() {
-                unknown_fields.push(format!("packages.{}.{}", name, field));
+                unknown_fields.push(format!("packages.{name}.{field}"));
             }
         }
 
@@ -167,10 +181,7 @@ impl Config {
     }
 
     pub fn build_bin_package(&self) -> bool {
-        match self.bin {
-            None => !self.semver_suffix,
-            Some(b) => b,
-        }
+        self.bin.unwrap_or(!self.semver_suffix)
     }
 
     pub fn overlay_dir(&self, config_path: Option<&Path>) -> Option<PathBuf> {
@@ -189,6 +200,12 @@ impl Config {
         self.whitelist.as_ref()
     }
 
+    pub fn repack_suffix(&self) -> Option<&str> {
+        self.repack_suffix
+            .as_deref()
+            .or_else(|| self.orig_tar_excludes().and(Some(DEFAULT_REPACK_SUFFIX)))
+    }
+
     pub fn maintainer(&self) -> &str {
         self.maintainer.as_str()
     }
@@ -243,14 +260,18 @@ impl Config {
         self.source.as_ref()?.skip_nocheck
     }
 
-    // Packages shortcuts
+    // Packages accessors
+
+    pub fn configured_packages(&'_ self) -> impl Iterator<Item = PackageKey<'_>> {
+        self.packages.keys().filter_map(|k| PackageKey::from_key(k))
+    }
 
     fn with_package<'a, T, F: FnOnce(&'a PackageOverride) -> Option<T>>(
         &'a self,
         key: PackageKey,
         f: F,
     ) -> Option<T> {
-        self.packages.get(&package_key_string(key)).and_then(f)
+        self.packages.get(&key.key_string()[..]).and_then(f)
     }
 
     pub fn package_section(&self, key: PackageKey) -> Option<&str> {
@@ -318,17 +339,16 @@ impl Config {
     }
 }
 
-pub fn package_field_for_feature<'a>(
-    get_field: &'a dyn Fn(PackageKey) -> Option<&'a Vec<String>>,
-    feature: PackageKey,
-    f_provides: &[&str],
-) -> Vec<String> {
+pub fn package_field_for_feature<'a, 'b, F: Fn(PackageKey) -> Option<&'a Vec<String>>>(
+    get_field: F,
+    feature: PackageKey<'b>,
+    f_provides: &'b [&'b str],
+) -> impl Iterator<Item = String> + use<'a, 'b, F> {
     Some(feature)
         .into_iter()
         .chain(f_provides.iter().map(|s| PackageKey::feature(s)))
         .flat_map(move |f| get_field(f).into_iter().flatten())
-        .map(|s| s.to_string())
-        .collect()
+        .map(ToString::to_string)
 }
 
 #[derive(Clone, Copy)]
@@ -336,32 +356,52 @@ pub enum PackageKey<'a> {
     Bin,
     BareLib,
     FeatureLib(&'a str),
+    Extra(&'a str),
 }
 
 impl<'a> PackageKey<'a> {
     pub fn feature(f: &'a str) -> PackageKey<'a> {
-        use self::PackageKey::*;
+        use self::PackageKey::{BareLib, FeatureLib};
         if f.is_empty() {
             BareLib
         } else {
             FeatureLib(f)
         }
     }
-}
 
-fn package_key_string(key: PackageKey) -> String {
-    use self::PackageKey::*;
-    match key {
-        Bin => "bin".to_string(),
-        BareLib => "lib".to_string(),
-        FeatureLib(feature) => format!("lib+{}", feature),
+    pub fn from_key(k: &'a str) -> Option<PackageKey<'a>> {
+        use self::PackageKey::{BareLib, Bin, Extra, FeatureLib};
+        Some(match k {
+            "bin" => Bin,
+            "lib" => BareLib,
+            _ => {
+                if let Some(feature) = k.strip_prefix("lib+") {
+                    FeatureLib(feature)
+                } else if let Some(package) = k.strip_prefix("extra+") {
+                    Extra(package)
+                } else {
+                    return None;
+                }
+            }
+        })
+    }
+
+    fn key_string(&self) -> Cow<'static, str> {
+        use self::PackageKey::{BareLib, Bin, Extra, FeatureLib};
+        match self {
+            Bin => "bin".into(),
+            BareLib => "lib".into(),
+            FeatureLib(feature) => format!("lib+{feature}").into(),
+            Extra(package) => format!("extra+{package}").into(),
+        }
     }
 }
 
 pub fn testing_ignore_debpolv() -> bool {
-    std::env::var("DEBCARGO_TESTING_IGNORE_DEBIAN_POLICY_VIOLATION") == Ok("1".to_string())
+    std::env::var_os("DEBCARGO_TESTING_IGNORE_DEBIAN_POLICY_VIOLATION").as_deref()
+        == Some(OsStr::new("1"))
 }
 
 pub fn testing_ruzt() -> bool {
-    std::env::var("DEBCARGO_TESTING_RUZT") == Ok("1".to_string())
+    std::env::var_os("DEBCARGO_TESTING_RUZT").as_deref() == Some(OsStr::new("1"))
 }
