@@ -5,7 +5,7 @@ use cargo::{
         Dependency, EitherManifest, FeatureValue, Manifest, Package, PackageId, Registry, SourceId,
         Summary, Target, TargetKind, Workspace,
     },
-    ops::{self, PackageOpts, Packages},
+    ops::{self, PackageMessageFormat, PackageOpts, Packages},
     sources::{
         source::{MaybePackage, QueryKind, Source},
         IndexSummary, RegistrySource, SourceConfigMap,
@@ -24,15 +24,15 @@ use semver::Version;
 use tar::Archive;
 use tempfile;
 
+use crate::config::testing_ignore_debpolv;
+use crate::errors::Result;
+use cargo::util::Filesystem;
 use std::collections::{BTreeMap, HashSet};
 use std::fs;
 use std::io::{self, Read};
 use std::path::Path;
 use std::{self, ffi::OsStr};
 
-use crate::config::testing_ignore_debpolv;
-use crate::errors::*;
-
 pub struct CrateInfo {
     // only used for to_registry_toml in extract_crate. DO NOT USE ELSEWHERE
     package: Package,
@@ -165,7 +165,7 @@ impl CrateInfo {
                     let mut package_id: Option<PackageId> = None;
                     loop {
                         match source.query(&dep, QueryKind::Exact, &mut |p| {
-                            package_id = Some(p.package_id())
+                            package_id = Some(p.package_id());
                         }) {
                             std::task::Poll::Ready(res) => {
                                 res?;
@@ -209,6 +209,9 @@ impl CrateInfo {
                     to_package: Packages::Default,
                     keep_going: false,
                     reg_or_index: None,
+                    dry_run: false,
+                    include_lockfile: false,
+                    fmt: PackageMessageFormat::Human,
                 };
                 ops::package(&workspace, &opts)?;
 
@@ -253,10 +256,10 @@ impl CrateInfo {
                 0,
                 false,
                 None,
-                context.frozen(),
-                context.locked(),
+                false,
+                false,
                 true, // offline
-                &context.target_dir()?.map(|x| x.into_path_unlocked()),
+                &context.target_dir()?.map(Filesystem::into_path_unlocked),
                 &[],
                 &[],
             )?;
@@ -367,8 +370,8 @@ impl CrateInfo {
         match *self.package_id().version() {
             Version {
                 major: 0, minor, ..
-            } => format!("0.{}", minor),
-            Version { major, .. } => format!("{}", major),
+            } => format!("0.{minor}"),
+            Version { major, .. } => format!("{major}"),
         }
     }
 
@@ -392,7 +395,7 @@ impl CrateInfo {
     }
 
     pub fn rust_version(&self) -> Option<String> {
-        self.manifest.rust_version().map(|v| v.to_string())
+        self.manifest.rust_version().map(ToString::to_string)
     }
 
     pub fn targets(&self) -> &[Target] {
@@ -452,7 +455,7 @@ impl CrateInfo {
         let mut deps = vec![];
         for dep in self.dependencies() {
             if dep.kind() == DepKind::Development {
-                deps.push(dep.clone())
+                deps.push(dep.clone());
             }
         }
         deps
@@ -513,7 +516,7 @@ impl CrateInfo {
         (summary, description)
     }
 
-    /// To be called before extract_crate.
+    /// To be called before `extract_crate`.
     pub fn set_includes_excludes(
         &mut self,
         excludes: Option<&Vec<String>>,
@@ -541,15 +544,14 @@ impl CrateInfo {
             return Ok(true);
         }
         let suspicious = match path.extension() {
-            Some(ext) => ext == "c" || ext == "a",
+            Some(ext) => ext == "c" || ext == "a" || ext == "S" || ext == "o" || ext == "so",
             _ => false,
         };
 
         let debian_dir_pattern = Pattern::new(&format!("{top_level}/debian/*")).unwrap();
         if debian_dir_pattern.matches_path(path) {
             return Err(format!(
-                "Suspicious file or directory, should probably be excluded: {:?}",
-                path
+                "Suspicious file or directory, should probably be excluded in debcargo.toml: {path:?}"
             ));
         }
 
@@ -562,8 +564,7 @@ impl CrateInfo {
                 Ok(false)
             } else {
                 Err(format!(
-                    "Suspicious file, should probably be excluded: {:?}",
-                    path
+                    "Suspicious file, should probably be excluded in debcargo.toml: {path:?}"
                 ))
             }
         } else {
@@ -571,12 +572,13 @@ impl CrateInfo {
         }
     }
 
-    pub fn extract_crate(&mut self, path: &Path) -> Result<bool> {
+    pub fn extract_crate(&mut self, path: &Path) -> Result<(bool, bool)> {
         let mut archive = Archive::new(GzDecoder::new(self.crate_file.file()));
         let tempdir = tempfile::Builder::new()
             .prefix("debcargo")
             .tempdir_in(".")?;
         let mut source_modified = false;
+        let mut manifest_normalized = false;
         let mut last_mtime = 0;
         let mut err = vec![];
 
@@ -656,7 +658,7 @@ impl CrateInfo {
                             "Cargo.toml references non-existing license_file path: {:?}",
                             full_referenced_path
                         );
-                        if let Some((_prefix, file)) = referenced_path.rsplit_once("/") {
+                        if let Some((_prefix, file)) = referenced_path.rsplit_once('/') {
                             if path.join(file).exists() {
                                 debcargo_info!("Replacing reference with '{}'", file);
                                 actual_toml = actual_toml.replace(referenced_path, file);
@@ -697,6 +699,9 @@ impl CrateInfo {
                 targets: Vec::new(),
                 cli_features: CliFeatures::new_all(true),
                 reg_or_index: None,
+                dry_run: false,
+                include_lockfile: false,
+                fmt: PackageMessageFormat::Human,
             };
             let files = ops::package(&ws, &opts)?;
             let file = files
@@ -707,9 +712,9 @@ impl CrateInfo {
             for entry in archive.entries()? {
                 let mut entry = entry?;
                 let entry_path = entry.path()?;
-                let components = entry_path.iter();
+                let mut components = entry_path.iter();
                 if components.clone().count() == 2
-                    && components.last() == Some(OsStr::new("Cargo.toml"))
+                    && components.next_back() == Some(OsStr::new("Cargo.toml"))
                 {
                     entry.unpack(&toml_path)?;
                     break;
@@ -718,26 +723,37 @@ impl CrateInfo {
             fs::write(path.join("Cargo.toml.orig"), orig_toml.as_bytes())?;
             fs::remove_dir_all(path.join("target"))?;
             source_modified = true;
+            manifest_normalized = true;
             // avoid lintian errors about package-contains-ancient-file
             // TODO: do we want to do this for unmodified tarballs? it would
             // force us to modify them, but otherwise we get that ugly warning
-            let last_mtime = FileTime::from_unix_time(last_mtime as i64, 0);
+            let last_mtime = FileTime::from_unix_time(i64::try_from(last_mtime)?, 0);
             set_file_times(toml_path, last_mtime, last_mtime)?;
             debcargo_info!("Cargo.toml manually canonicalized!");
         }
-        Ok(source_modified)
+        Ok((source_modified, manifest_normalized))
     }
 }
 
 /// Collect information about the dependency structure of features and
 /// their external crate dependencies, in a simple output format.
 pub fn all_dependencies_and_features(manifest: &Manifest) -> CrateDepInfo {
+    all_dependencies_and_features_filtered(manifest, false)
+}
+
+/// Collect information about the dependency structure of features and
+/// their external crate dependencies, in a simple output format.
+/// If `include_dev_dependencies` is true, dev-dependencies will be included.
+pub fn all_dependencies_and_features_filtered(
+    manifest: &Manifest,
+    include_dev_dependencies: bool,
+) -> CrateDepInfo {
     use cargo::core::dependency::DepKind;
 
     let mut deps_by_name: BTreeMap<&str, Vec<&Dependency>> = BTreeMap::new();
     for dep in manifest.dependencies() {
         // we treat build-dependencies also as dependencies in Debian
-        if dep.kind() != DepKind::Development {
+        if include_dev_dependencies || dep.kind() != DepKind::Development {
             let s = dep.name_in_toml().as_str();
             deps_by_name.entry(s).or_default().push(dep);
         }
@@ -751,11 +767,11 @@ pub fn all_dependencies_and_features(manifest: &Manifest) -> CrateDepInfo {
         let mut feature_deps: Vec<&'static str> = vec![];
         let mut other_deps: Vec<Dependency> = Vec::new();
         for dep in deps {
-            use self::FeatureValue::*;
+            use self::FeatureValue::{Dep, DepFeature, Feature};
             match dep {
                 // another feature is a dependency
                 Feature(dep_feature) => {
-                    feature_deps.push(InternedString::new(dep_feature).as_str())
+                    feature_deps.push(InternedString::new(dep_feature).as_str());
                 }
                 // another package is a dependency
                 Dep { dep_name } => {
@@ -770,43 +786,41 @@ pub fn all_dependencies_and_features(manifest: &Manifest) -> CrateDepInfo {
                     dep_feature,
                     ..
                 } => {
-                    match deps_by_name.get(dep_name.as_str()) {
-                        // unwrap is ok, valid Cargo.toml files must have this
-                        Some(dd) => {
-                            for &dep in dd {
-                                let mut dep = dep.clone();
-                                let mut features: Vec<InternedString> =
-                                    vec![InternedString::new(dep_feature)];
-                                features.extend(dep.features());
-                                dep.set_features(features);
-                                dep.set_default_features(false);
-                                other_deps.push(dep);
-                            }
+                    if let Some(dd) = deps_by_name.get(dep_name.as_str()) {
+                        for &dep in dd {
+                            let mut dep = dep.clone();
+                            let mut features: Vec<InternedString> =
+                                vec![InternedString::new(dep_feature)];
+                            features.extend(dep.features());
+                            dep.set_features(features);
+                            dep.set_default_features(false);
+                            other_deps.push(dep);
                         }
-                        None => {
-                            let mut expected = false;
-                            for dep in manifest.dependencies() {
-                                if dep.kind() == DepKind::Development {
-                                    let s = dep.name_in_toml().as_str();
-                                    if s == dep_name.as_str() {
-                                        expected = true;
-                                    }
+                    } else {
+                        let mut expected = false;
+                        for dep in manifest.dependencies() {
+                            if dep.kind() == DepKind::Development {
+                                let s = dep.name_in_toml().as_str();
+                                if s == dep_name.as_str() {
+                                    expected = true;
                                 }
                             }
-                            if expected {
-                                debcargo_warn!(
-                                    "Ignoring \"{}\" feature \"{}\" as it depends on a \
+                        }
+                        if expected {
+                            debcargo_warn!(
+                                "Ignoring \"{}\" feature \"{}\" as it depends on a \
                                      dev-dependency \"{}\"",
-                                    manifest.package_id(),
-                                    feature,
-                                    dep_name
-                                );
-                            } else {
-                                panic!(
-                                    "failed to account for dependency \"{}\" of \"{}\" feature \"{}\"",
-                                    dep_name, manifest.package_id(), feature
-                                );
-                            }
+                                manifest.package_id(),
+                                feature,
+                                dep_name
+                            );
+                        } else {
+                            panic!(
+                                "failed to account for dependency \"{}\" of \"{}\" feature \"{}\"",
+                                dep_name,
+                                manifest.package_id(),
+                                feature
+                            );
                         }
                     }
                 }
@@ -824,7 +838,7 @@ pub fn all_dependencies_and_features(manifest: &Manifest) -> CrateDepInfo {
     for deps in deps_by_name.values() {
         for &dep in deps {
             if !dep.is_optional() {
-                deps_required.push(dep.clone())
+                deps_required.push(dep.clone());
             }
         }
     }
