@@ -8,7 +8,10 @@ use semver::Version;
 use textwrap::fill;
 
 use crate::config::{self, Config, PackageKey};
-use crate::errors::*;
+use crate::errors::Result;
+
+/// Suffix for Rust development packages
+pub const DEV_SUFFIX: &str = "-dev";
 
 #[derive(Default, Debug)]
 pub struct BuildDeps {
@@ -19,8 +22,8 @@ pub struct BuildDeps {
 
 pub struct Source {
     name: String,
+    version: String,
     section: String,
-    priority: String,
     maintainer: String,
     uploaders: Vec<String>,
     standards: String,
@@ -75,7 +78,6 @@ impl fmt::Display for Source {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         writeln!(f, "Source: {}", self.name)?;
         writeln!(f, "Section: {}", self.section)?;
-        writeln!(f, "Priority: {}", self.priority)?;
         if !self.build_deps.build_depends.is_empty() {
             writeln!(
                 f,
@@ -116,8 +118,11 @@ impl fmt::Display for Source {
         //   b) "utf-8" crate at version latest with semver_suffix = false.
         // dh-cargo assumes (a) which is wrong for the "utf-8" crate
         writeln!(f, "X-Cargo-Crate: {}", self.crate_name)?;
+        // Needed to get the plain upstream version for encoding it into the
+        // vendor/registry path used as install directory
+        writeln!(f, "X-Cargo-Crate-Version: {}", self.version)?;
         if let Some(ref rrr) = self.requires_root {
-            writeln!(f, "Rules-Requires-Root: {}", rrr)?;
+            writeln!(f, "Rules-Requires-Root: {rrr}")?;
         }
 
         Ok(())
@@ -129,11 +134,11 @@ impl fmt::Display for Package {
         writeln!(f, "Package: {}", self.name)?;
         writeln!(f, "Architecture: {}", self.arch)?;
         if let Some(ref multi_arch) = self.multi_arch {
-            writeln!(f, "Multi-Arch: {}", multi_arch)?;
+            writeln!(f, "Multi-Arch: {multi_arch}")?;
         }
 
         if let Some(section) = &self.section {
-            writeln!(f, "Section: {}", section)?;
+            writeln!(f, "Section: {section}")?;
         }
 
         if !self.depends.is_empty() {
@@ -159,7 +164,7 @@ impl fmt::Display for Package {
         }
 
         for line in &self.extra_lines {
-            writeln!(f, "{}", line)?;
+            writeln!(f, "{line}")?;
         }
 
         self.write_description(f)
@@ -169,7 +174,7 @@ impl fmt::Display for Package {
 impl fmt::Display for PkgTest {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let extra_args = if self.extra_test_args.is_empty() {
-            "".into()
+            String::new()
         } else {
             format!(" {}", self.extra_test_args.join(" "))
         };
@@ -186,21 +191,20 @@ impl fmt::Display for PkgTest {
         let default_deps = if cargo_bug_fixed { &self.name } else { "@" };
 
         let depends = if self.depends.is_empty() {
-            "".into()
+            String::new()
         } else {
             format!(", {}", self.depends.join(", "))
         };
-        writeln!(f, "Depends: dh-cargo (>= 31){}, {}", depends, default_deps)?;
+        writeln!(f, "Depends: dh-cargo (>= 33~){depends}, {default_deps}")?;
 
         let restricts = if self.extra_restricts.is_empty() {
-            "".into()
+            String::new()
         } else {
             format!(", {}", self.extra_restricts.join(", "))
         };
         writeln!(
             f,
-            "Restrictions: allow-stderr, skip-not-installable{}",
-            restricts,
+            "Restrictions: allow-stderr, skip-not-installable{restricts}",
         )?;
         if !self.architecture.is_empty() {
             writeln!(f, "Architecture: {}", self.architecture.join(" "))?;
@@ -224,6 +228,7 @@ impl Source {
         basename: &str,
         name_suffix: Option<&str>,
         crate_name: &str,
+        crate_version: &str,
         home: &str,
         lib: bool,
         maintainer: String,
@@ -233,29 +238,24 @@ impl Source {
     ) -> Result<Source> {
         let pkgbase = match name_suffix {
             None => basename.to_string(),
-            Some(suf) => format!("{}{}", basename, suf),
+            Some(suf) => format!("{basename}{suf}"),
         };
         let section = if lib {
             "rust"
         } else {
             "FIXME-IN-THE-SOURCE-SECTION"
         };
-        let priority = "optional".to_string();
-        let vcs_browser = format!(
-            "https://salsa.debian.org/rust-team/debcargo-conf/tree/master/src/{}",
-            pkgbase
-        );
-        let vcs_git = format!(
-            "https://salsa.debian.org/rust-team/debcargo-conf.git [src/{}]",
-            pkgbase
-        );
+        let vcs_browser =
+            format!("https://salsa.debian.org/rust-team/debcargo-conf/tree/master/src/{pkgbase}");
+        let vcs_git =
+            format!("https://salsa.debian.org/rust-team/debcargo-conf.git [src/{pkgbase}]");
         Ok(Source {
             name: dsc_name(&pkgbase),
+            version: crate_version.to_string(),
             section: section.to_string(),
-            priority,
             maintainer,
             uploaders,
-            standards: "4.7.2".to_string(),
+            standards: "4.7.3".to_string(),
             build_deps,
             vcs_git,
             vcs_browser,
@@ -299,10 +299,7 @@ impl Source {
                 .flatten()
                 .map(String::to_string),
         );
-        let bdeps_ex = config
-            .build_depends_excludes()
-            .map(Vec::as_slice)
-            .unwrap_or(&[]);
+        let bdeps_ex: &[String] = config.build_depends_excludes().map_or(&[], Vec::as_slice);
         self.build_deps
             .build_depends
             .retain(|x| !bdeps_ex.contains(x));
@@ -326,7 +323,6 @@ impl Source {
         if let Some(vcs_browser) = config.vcs_browser() {
             self.vcs_browser = vcs_browser.to_string();
         }
-
     }
 }
 
@@ -340,6 +336,26 @@ impl Package {
         }
     }
 
+    fn deb_feature2(p: &str, f: &str) -> String {
+        format!(
+            "{} (= ${{binary:Version}})",
+            match f {
+                "" => deb_name(p),
+                _ => deb_feature_name(p, f),
+            }
+        )
+    }
+    fn deb_feature(f: &str, pkgbase: &str) -> String {
+        Package::deb_feature2(pkgbase, f)
+    }
+
+    fn filter_provides(x: &[&str], f_provides: &[&str], pkgbase: &str) -> Vec<String> {
+        x.iter()
+            .filter(|f| !f_provides.contains(f))
+            .map(|f| Package::deb_feature(f, pkgbase))
+            .collect()
+    }
+
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         basename: &str,
@@ -348,36 +364,22 @@ impl Package {
         summary: Description,
         description: Description,
         feature: Option<&str>,
-        f_deps: Vec<&str>,
+        f_deps: &[&str],
         o_deps: Vec<String>,
-        f_provides: Vec<&str>,
-        f_recommends: Vec<&str>,
-        f_suggests: Vec<&str>,
+        f_provides: &[&str],
+        f_recommends: &[&str],
+        f_suggests: &[&str],
     ) -> Result<Package> {
         let pkgbase = match name_suffix {
             None => basename.to_string(),
-            Some(suf) => format!("{}{}", basename, suf),
-        };
-        let deb_feature2 = &|p: &str, f: &str| {
-            format!(
-                "{} (= ${{binary:Version}})",
-                match f {
-                    "" => deb_name(p),
-                    _ => deb_feature_name(p, f),
-                }
-            )
-        };
-        let deb_feature = &|f: &str| deb_feature2(&pkgbase, f);
-
-        let filter_provides = &|x: Vec<&str>| {
-            x.into_iter()
-                .filter(|f| !f_provides.contains(f))
-                .map(deb_feature)
-                .collect()
+            Some(suf) => format!("{basename}{suf}"),
         };
         let (recommends, suggests) = match feature {
             Some(_) => (vec![], vec![]),
-            None => (filter_provides(f_recommends), filter_provides(f_suggests)),
+            None => (
+                Package::filter_provides(f_recommends, f_provides, &pkgbase),
+                Package::filter_provides(f_suggests, f_provides, &pkgbase),
+            ),
         };
 
         // Provides for all possible versions, see:
@@ -385,22 +387,22 @@ impl Package {
         // https://wiki.debian.org/Teams/RustPackaging/Policy#Package_provides
         let mut provides = vec![];
         let version_suffixes = [
-            "".to_string(),
+            String::new(),
             format!("-{}", version.major),
             format!("-{}.{}", version.major, version.minor),
             format!("-{}.{}.{}", version.major, version.minor, version.patch),
         ];
-        for suffix in version_suffixes.iter() {
+        for suffix in &version_suffixes {
             // don't provide unversioned variants in semver-suffix packages
             if name_suffix.is_some() && suffix.is_empty() {
                 continue;
-            };
+            }
 
-            let p = format!("{}{}", basename, suffix);
-            provides.push(deb_feature2(&p, feature.unwrap_or("")));
-            provides.extend(f_provides.iter().map(|f| deb_feature2(&p, f)));
+            let p = format!("{basename}{suffix}");
+            provides.push(Package::deb_feature2(&p, feature.unwrap_or("")));
+            provides.extend(f_provides.iter().map(|f| Package::deb_feature2(&p, f)));
         }
-        let provides_self = deb_feature(feature.unwrap_or(""));
+        let provides_self = Package::deb_feature(feature.unwrap_or(""), &pkgbase);
         // rust dropped Vec::remove_item for annoying reasons, the below is
         // an unofficialy recommended replacement from the RFC #40062
         let i = provides.iter().position(|x| *x == *provides_self);
@@ -411,9 +413,9 @@ impl Package {
             // in dh-cargo we symlink /usr/share/doc/{$feature => $main} pkg
             // so we always need this direct dependency, even if the feature
             // only indirectly depends on the bare library via another
-            depends.push(deb_feature(""));
+            depends.push(Package::deb_feature("", &pkgbase));
         }
-        depends.extend(f_deps.into_iter().map(deb_feature));
+        depends.extend(f_deps.iter().map(|f| Package::deb_feature(f, &pkgbase)));
         depends.extend(o_deps);
         let mut breaks = vec![];
         let mut replaces = vec![];
@@ -479,7 +481,7 @@ impl Package {
         let (name, mut provides) = match name_suffix {
             None => (basename.to_string(), vec![]),
             Some(suf) => (
-                format!("{}{}", basename, suf),
+                format!("{basename}{suf}"),
                 vec![format!("{} (= ${{binary:Version}})", basename)],
             ),
         };
@@ -488,7 +490,7 @@ impl Package {
             name,
             arch: "any".to_string(),
             multi_arch: None,
-            section: section.map(|s| s.to_string()),
+            section: section.map(ToString::to_string),
             depends: vec![
                 "${misc:Depends}".to_string(),
                 "${shlibs:Depends}".to_string(),
@@ -509,6 +511,25 @@ impl Package {
         }
     }
 
+    pub fn new_extra(name: String) -> Self {
+        Package {
+            name,
+            arch: String::default(),
+            multi_arch: Option::default(),
+            section: Option::default(),
+            depends: Vec::default(),
+            recommends: Vec::default(),
+            suggests: Vec::default(),
+            provides: Vec::default(),
+            breaks: Vec::default(),
+            replaces: Vec::default(),
+            conflicts: Vec::default(),
+            summary: Description::new(String::default(), String::default()),
+            description: Description::new(String::default(), String::default()),
+            extra_lines: Vec::default(),
+        }
+    }
+
     pub fn name(&self) -> &str {
         self.name.as_str()
     }
@@ -521,9 +542,9 @@ impl Package {
             if line.is_empty() {
                 writeln!(out, " .")?;
             } else if line.starts_with("- ") {
-                writeln!(out, "  {}", line)?;
+                writeln!(out, "  {line}")?;
             } else {
-                writeln!(out, " {}", line)?;
+                writeln!(out, " {line}")?;
             }
         }
         Ok(())
@@ -538,56 +559,58 @@ impl Package {
         }
     }
 
-    pub fn apply_overrides(&mut self, config: &Config, key: PackageKey, f_provides: Vec<&str>) {
+    pub fn apply_overrides(&mut self, config: &Config, key: PackageKey, f_provides: &[&str]) {
         if let Some(section) = config.package_section(key) {
             self.section = Some(section.to_string());
         }
         self.summary
-            .apply_overrides(&config.summary, config.package_summary(key));
-        self.description
-            .apply_overrides(&config.description, config.package_description(key));
+            .apply_overrides(config.summary.as_deref(), config.package_summary(key));
+        self.description.apply_overrides(
+            config.description.as_deref(),
+            config.package_description(key),
+        );
 
         self.depends.extend(config::package_field_for_feature(
-            &|x| config.package_depends(x),
+            |x| config.package_depends(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.recommends.extend(config::package_field_for_feature(
-            &|x| config.package_recommends(x),
+            |x| config.package_recommends(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.suggests.extend(config::package_field_for_feature(
-            &|x| config.package_suggests(x),
+            |x| config.package_suggests(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.provides.extend(config::package_field_for_feature(
-            &|x| config.package_provides(x),
+            |x| config.package_provides(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.breaks.extend(config::package_field_for_feature(
-            &|x| config.package_breaks(x),
+            |x| config.package_breaks(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.replaces.extend(config::package_field_for_feature(
-            &|x| config.package_replaces(x),
+            |x| config.package_replaces(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.conflicts.extend(config::package_field_for_feature(
-            &|x| config.package_conflicts(x),
+            |x| config.package_conflicts(x),
             key,
-            &f_provides,
+            f_provides,
         ));
         self.extra_lines.extend(
             config
                 .package_extra_lines(key)
                 .into_iter()
                 .flatten()
-                .map(|s| s.to_string()),
+                .map(ToString::to_string),
         );
         if let Some(architecture) = config.package_architecture(key) {
             self.arch = architecture.join(" ");
@@ -599,12 +622,12 @@ impl Package {
 }
 
 impl Description {
-    fn apply_overrides(&mut self, global: &Option<String>, per_package: Option<&str>) {
+    fn apply_overrides(&mut self, global: Option<&str>, per_package: Option<&str>) {
         if let Some(per_package) = per_package {
             self.prefix = per_package.to_string();
-            self.suffix = "".to_string();
-        } else if let Some(global) = &global {
-            self.prefix = global.into();
+            self.suffix = String::new();
+        } else if let Some(global) = global {
+            self.prefix = global.to_owned();
         }
     }
 }
@@ -621,9 +644,9 @@ impl PkgTest {
         crate_name: &str,
         feature: &str,
         version: &str,
-        extra_test_args: Vec<&str>,
+        extra_test_args: &[&str],
         depends: &[String],
-        extra_restricts: Vec<&str>,
+        extra_restricts: &[&str],
         architecture: &[&str],
     ) -> Result<PkgTest> {
         Ok(PkgTest {
@@ -631,10 +654,10 @@ impl PkgTest {
             crate_name: crate_name.to_string(),
             feature: feature.to_string(),
             version: version.to_string(),
-            extra_test_args: extra_test_args.iter().map(|x| x.to_string()).collect(),
+            extra_test_args: extra_test_args.iter().map(ToString::to_string).collect(),
             depends: depends.to_vec(),
-            extra_restricts: extra_restricts.iter().map(|x| x.to_string()).collect(),
-            architecture: architecture.iter().map(|x| x.to_string()).collect(),
+            extra_restricts: extra_restricts.iter().map(ToString::to_string).collect(),
+            architecture: architecture.iter().map(ToString::to_string).collect(),
         })
     }
 }
@@ -642,11 +665,14 @@ impl PkgTest {
 /// Translates a semver into a Debian-format upstream version.
 /// Omits the build metadata, and uses a ~ before the prerelease version so it
 /// compares earlier than the subsequent release.
-pub fn deb_upstream_version(v: &Version) -> String {
+pub fn deb_upstream_version(v: &Version, repack: Option<&str>) -> String {
     let mut s = format!("{}.{}.{}", v.major, v.minor, v.patch);
     if !v.pre.is_empty() {
         write!(s, "~{}", v.pre.as_str()).unwrap();
     }
+    if let Some(repack_suffix) = repack {
+        write!(s, "+{repack_suffix}").unwrap();
+    }
     s
 }
 
@@ -659,15 +685,21 @@ pub fn dsc_name(name: &str) -> String {
 }
 
 pub fn deb_name(name: &str) -> String {
-    format!("{}-{}-dev", Package::pkg_prefix(), base_deb_name(name))
+    format!(
+        "{}-{}{}",
+        Package::pkg_prefix(),
+        base_deb_name(name),
+        DEV_SUFFIX
+    )
 }
 
 pub fn deb_feature_name(name: &str, feature: &str) -> String {
     format!(
-        "{}-{}+{}-dev",
+        "{}-{}+{}{}",
         Package::pkg_prefix(),
         base_deb_name(name),
-        base_deb_name(feature)
+        base_deb_name(feature),
+        DEV_SUFFIX
     )
 }
 
@@ -681,8 +713,9 @@ fn get_envs(keys: &[&str]) -> Result<Option<String>> {
                 return Ok(Some(val));
             }
             Err(e @ VarError::NotUnicode(_)) => {
-                return Err(Error::from(e)
-                    .context(format!("Environment variable ${} not valid UTF-8", key)));
+                return Err(
+                    Error::from(e).context(format!("Environment variable ${key} not valid UTF-8"))
+                );
             }
             Err(VarError::NotPresent) => {}
         }
@@ -704,7 +737,7 @@ pub fn get_deb_author() -> Result<String> {
     let email = get_envs(&["DEBEMAIL", "EMAIL"])?.ok_or_else(|| {
         format_err!("Unable to determine your email; please set $DEBEMAIL or $EMAIL")
     })?;
-    Ok(format!("{} <{}>", name, email))
+    Ok(format!("{name} <{email}>"))
 }
 
 #[cfg(test)]
