@@ -14,7 +14,7 @@ use std::fs;
 use std::io::{BufRead, BufReader, Read};
 use std::path::Path;
 
-use crate::errors::*;
+use crate::errors::Result;
 
 const DEB_COPYRIGHT_FORMAT: &str = "\
      https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/";
@@ -58,19 +58,27 @@ pub struct DebCopyright {
     upstream: UpstreamInfo,
     files: Vec<Files>,
     licenses: Vec<License>,
+    exclusions: Vec<String>,
 }
 
 impl fmt::Display for DebCopyright {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         writeln!(f, "Format: {}", self.format)?;
         write!(f, "{}", self.upstream)?;
+        if !self.exclusions.is_empty() {
+            write!(f, "Files-Excluded:")?;
+            for excluded in &self.exclusions {
+                write!(f, "\n {excluded}")?;
+            }
+            writeln!(f)?;
+        }
 
         for file in &self.files {
-            write!(f, "\n{}", file)?;
+            write!(f, "\n{file}")?;
         }
 
         for license in &self.licenses {
-            write!(f, "\n{}", license)?;
+            write!(f, "\n{license}")?;
         }
 
         Ok(())
@@ -78,12 +86,13 @@ impl fmt::Display for DebCopyright {
 }
 
 impl DebCopyright {
-    fn new(u: UpstreamInfo, f: &[Files], l: &[License]) -> DebCopyright {
+    fn new(u: UpstreamInfo, f: &[Files], l: &[License], e: &[String]) -> DebCopyright {
         DebCopyright {
             format: DEB_COPYRIGHT_FORMAT.to_string(),
             upstream: u,
             files: f.to_vec(),
             licenses: l.to_vec(),
+            exclusions: e.to_vec(),
         }
     }
 }
@@ -96,7 +105,7 @@ impl fmt::Display for UpstreamInfo {
             writeln!(f)?;
         }
         for contact in &self.contacts {
-            writeln!(f, " {}", contact)?;
+            writeln!(f, " {contact}")?;
         }
         if !self.source.is_empty() {
             writeln!(f, "Source: {}", self.source)?;
@@ -125,7 +134,7 @@ impl fmt::Display for Files {
             writeln!(f)?;
         }
         for copyright in &self.copyright {
-            writeln!(f, " {}", copyright)?;
+            writeln!(f, " {copyright}")?;
         }
         writeln!(f, "License: {}", self.license)?;
         if !self.comment.is_empty() {
@@ -141,7 +150,7 @@ impl Files {
         assert!(!notice.is_empty());
         Files {
             files: name.to_string(),
-            copyright: notice.iter().map(|s| s.to_string()).collect(),
+            copyright: notice.iter().map(ToString::to_string).collect(),
             license: license.to_string(),
             comment: comment.to_string(),
         }
@@ -283,13 +292,14 @@ fn get_licenses(license: &str) -> Result<Vec<License>> {
             .trim_end_matches("-or-later")
             .to_string();
         let text = match known_licenses.get(lname.as_str()) {
-            Some(s) => s.to_string(),
-            None => "FIXME (overlay): Unrecognized crate license, please find the \
+            Some(s) => s,
+            None => {
+                "FIXME (overlay): Unrecognized crate license, please find the \
                      full license text in the rest of the crate source code and \
                      copy-paste it here"
-                .to_string(),
+            }
         };
-        licenses.insert(ls.trim().to_string(), text);
+        licenses.insert(ls.trim().to_string(), text.to_string());
     }
 
     let mut lblocks: Vec<License> = Vec::new();
@@ -327,13 +337,14 @@ fn copyright_fromgit(repo_url: &str) -> Result<String> {
         .year();
 
     let notice = match first_year.cmp(latest_year) {
-        Ordering::Equal => format!("{}", first_year),
-        _ => format!("{}-{},", first_year, latest_year),
+        Ordering::Equal => format!("{first_year}"),
+        _ => format!("{first_year}-{latest_year},"),
     };
 
     Ok(notice)
 }
 
+#[expect(clippy::too_many_arguments)]
 pub fn debian_copyright(
     srcdir: &Path,
     manifest: &manifest::Manifest,
@@ -342,6 +353,7 @@ pub fn debian_copyright(
     uploaders: &[&str],
     year_range: (i32, i32),
     guess_harder: bool,
+    exclusions: &[String],
 ) -> Result<DebCopyright> {
     let meta = manifest.metadata().clone();
     let repository = meta.repository.unwrap_or_default();
@@ -359,7 +371,7 @@ pub fn debian_copyright(
     let upstream = UpstreamInfo::new(manifest.name().to_string(), authors, &repository);
 
     let mut licenses: Vec<License> = Vec::new();
-    let mut crate_license: String = "".to_string();
+    let mut crate_license: String = String::new();
 
     if let Some(ref license_file_name) = meta.license_file {
         let license_file = manifest_path.with_file_name(license_file_name);
@@ -390,12 +402,12 @@ pub fn debian_copyright(
 
     let (y0, y1) = year_range;
     let years = if y0 == y1 {
-        format!("{}", y0)
+        format!("{y0}")
     } else {
-        format!("{}-{}", y0, y1)
+        format!("{y0}-{y1}")
     };
     let mut deb_notice = vec![format!("{} {}", years, maintainer)];
-    deb_notice.extend(uploaders.iter().map(|s| format!("{} {}", years, s)));
+    deb_notice.extend(uploaders.iter().map(|s| format!("{years} {s}")));
     files.push(Files::new("debian/*", &deb_notice, &crate_license, ""));
 
     // Insert catch all block as the first block of copyright file. Capture
@@ -421,7 +433,7 @@ pub fn debian_copyright(
         _ => meta
             .authors
             .iter()
-            .map(|s| format!("{} {}", years, s))
+            .map(|s| format!("{years} {s}"))
             .collect(),
     };
     let comment = concat!(
@@ -435,7 +447,7 @@ pub fn debian_copyright(
         Files::new("*", notice.as_slice(), &crate_license, &fill(comment, 79)),
     );
 
-    Ok(DebCopyright::new(upstream, &files, &licenses))
+    Ok(DebCopyright::new(upstream, &files, &licenses, exclusions))
 }
 
 #[cfg(test)]
