@@ -7,14 +7,14 @@ pub use anyhow::format_err;
 macro_rules! debcargo_info {
     ($e:expr) => {
         {
-            use ansi_term::Colour::Green;
+            use nu_ansi_term::Color::Green;
             eprintln!("{}", Green.paint($e));
         }
     };
 
     ($fmt:expr, $( $arg:tt)+) => {
         {
-            use ansi_term::Colour::Green;
+            use nu_ansi_term::Color::Green;
             let print_string = format!($fmt, $($arg)+);
             eprintln!("{}", Green.paint(print_string));
         }
@@ -25,15 +25,15 @@ macro_rules! debcargo_info {
 macro_rules! debcargo_warn {
     ($e:expr) => {
         {
-            use ansi_term::Colour::RGB;
-            eprintln!("{}", RGB(255,165,0).bold().paint($e));
+            use nu_ansi_term::Color::Rgb;
+            eprintln!("{}", Rgb(255,165,0).bold().paint($e));
         }
     };
 
     ($fmt:expr, $( $arg:tt)+) => {
         {
-            use ansi_term::Colour::RGB;
-            let print_string = RGB(255,165,0).bold().paint(format!($fmt, $($arg)+));
+            use nu_ansi_term::Color::Rgb;
+            let print_string = Rgb(255,165,0).bold().paint(format!($fmt, $($arg)+));
             eprintln!("{}", print_string);
         }
     };
