@@ -132,14 +132,14 @@ pub fn show_vec<'a, T>(it: impl IntoIterator<Item = &'a T>) -> String
 where
     T: fmt::Display + 'a,
 {
-    show_vec_with(it, std::string::ToString::to_string)
+    show_vec_with(it, ToString::to_string)
 }
 
 pub fn expect_success(cmd: &mut Command, err: &str) -> Result<(), anyhow::Error> {
     match cmd.status() {
         Ok(status) if status.success() => Ok(()),
-        Ok(_) => bail!("{}", err),
-        Err(e) => bail!("{}\n{}", err, e),
+        Ok(_) => bail!("{err}"),
+        Err(e) => bail!("{err}\n{e}"),
     }
 }
 
@@ -183,7 +183,7 @@ pub(crate) fn get_transitive_val<
         let mut candidates = Vec::new();
         for par in getparents(key).into_iter().flatten() {
             if let Some(v) = get_transitive_val(getparents, f, *par)? {
-                candidates.push((*par, v))
+                candidates.push((*par, v));
             }
         }
         if candidates.is_empty() {
@@ -259,7 +259,7 @@ where
 
 pub fn topo_sort<V>(
     seed: impl IntoIterator<Item = V>,
-    succ: BTreeMap<V, BTreeSet<V>>,
+    succ: &BTreeMap<V, BTreeSet<V>>,
     mut pred: BTreeMap<V, BTreeSet<V>>,
 ) -> Result<Vec<V>, BTreeMap<V, BTreeSet<V>>>
 where
@@ -279,9 +279,9 @@ where
         }
     }
     pred.retain(|_, v| !v.is_empty());
-    if !pred.is_empty() {
-        Err(pred)
-    } else {
+    if pred.is_empty() {
         Ok(sort)
+    } else {
+        Err(pred)
     }
 }
